@@ -25,12 +25,12 @@
 #
 ##############################################################################
 
-import hashlib
 import json
 import os
 import signal
 import subprocess
 import sys
+import textwrap
 import urlparse
 
 from slapos.recipe.librecipe import GenericSlapRecipe
@@ -87,14 +87,17 @@ class Recipe(GenericSlapRecipe, Notify, Callback):
     url = entry.get('url')
     if not url:
       raise ValueError('Missing URL parameter for PBS recipe')
-
-    # We assume that thanks to sha512 there's no collisions
-    url_hash = hashlib.sha512(url).hexdigest()
-    name_hash = hashlib.sha512(entry['name']).hexdigest()
-
-    promise_path = os.path.join(self.options['promises-directory'],
-                                url_hash)
     parsed_url = urlparse.urlparse(url)
+
+    slave_type = entry['type']
+    if not slave_type in ['pull', 'push']:
+      raise ValueError('type parameter must be either pull or push.')
+
+    slave_id = entry['notification-id']
+
+    print 'Processing PBS slave %s with type %s' % (slave_id, slave_type)
+
+    promise_path = os.path.join(self.options['promises-directory'], slave_id)
     promise_dict = self.promise_base_dict.copy()
     promise_dict.update(user=parsed_url.username,
                         host=parsed_url.hostname,
@@ -104,72 +107,109 @@ class Recipe(GenericSlapRecipe, Notify, Callback):
                                       promise_dict)
     path_list.append(promise)
 
-
     host = parsed_url.hostname
     known_hosts_file[host] = entry['server-key']
 
+    notifier_wrapper_path = os.path.join(self.options['wrappers-directory'], slave_id)
+    rdiff_wrapper_path = notifier_wrapper_path + '_raw'
+
+    # Create the rdiff-backup wrapper
+    # It is useful to separate it from the notifier so that we can run it
+    # Manually.
+    rdiffbackup_parameter_list = []
+
     # XXX use -y because the host might not yet be in the
     #     trusted hosts file until the next time slapgrid is run.
-
-    remote_schema = '%(ssh)s -y -p %%s %(user)s@%(host)s' % \
-      {
-        'ssh': self.options['sshclient-binary'],
-        'user': parsed_url.username,
-        'host': parsed_url.hostname,
-      }
-
-    parameters = ['--remote-schema', remote_schema]
-
+    rdiffbackup_remote_schema = '%(ssh)s -y -p %%s %(user)s@%(host)s' % {
+            'ssh': self.options['sshclient-binary'],
+            'user': parsed_url.username,
+            'host': parsed_url.hostname,
+    }
     remote_directory = '%(port)s::%(path)s' % {'port': parsed_url.port,
                                                'path': parsed_url.path}
+    local_directory = self.createDirectory(self.options['directory'], entry['name'])
 
-    local_directory = self.createDirectory(self.options['directory'],
-                                           name_hash)
+    if slave_type == 'push':
+      # Create a simple rdiff-backup wrapper that will push
+      rdiffbackup_parameter_list.extend(['--remote-schema', rdiffbackup_remote_schema])
+      rdiffbackup_parameter_list.extend(['--restore-as-of', 'now'])
+      rdiffbackup_parameter_list.append('--force')
+      rdiffbackup_parameter_list.append(local_directory)
+      rdiffbackup_parameter_list.append(remote_directory)
+      comments = ['', 'Push data to a PBS *-import instance.', '']
+      rdiff_wrapper = self.createWrapper(
+          name=rdiff_wrapper_path,
+          command=self.options['rdiffbackup-binary'],
+          parameters=rdiffbackup_parameter_list,
+          comments=comments,
+          pidfile=os.path.join(self.options['run-directory'], '%s_raw.pid' % slave_id),
+      )
+    elif slave_type == 'pull':
+      # Wrap rdiff-backup call into a script that checks consistency of backup
+      # We need to manually escape the remote schema
+      rdiffbackup_parameter_list.extend(['--remote-schema', '"%s"' % rdiffbackup_remote_schema])
+      rdiffbackup_parameter_list.append(remote_directory)
+      rdiffbackup_parameter_list.append(local_directory)
+      comments = ['', 'Pull data from a PBS *-export instance.', '']
+      rdiff_wrapper_template = textwrap.dedent("""\
+          #!/bin/sh
+          # %(comment)s
+          RDIFF_BACKUP="%(rdiffbackup_binary)s"
+          $RDIFF_BACKUP %(rdiffbackup_parameter)s
+          if [ ! $? -eq 0 ]; then
+              # Check the backup, go to the last consistent backup, so that next
+              # run will be okay.
+              echo "Checking backup directory..."
+              $RDIFF_BACKUP --check-destination-dir %(local_directory)s
+              if [ ! $? -eq 0 ]; then
+                  # Here, two possiblities:
+                  # * The first backup failed. It is safe to remove it since there is nothing valuable there.
+                  # * The backup has been complete, but is now in a really weird state. Not safe to remove it.
+                  echo "Impossible to check backup: we move it to a safe place."
+                  # XXX: bang
+                  mv %(local_directory)s %(local_directory)s.$(date +%%s)
+              fi
+          else
+              # Everything's okay, cleaning up...
+              $RDIFF_BACKUP --remove-older-than %(remove_backup_older_than)s --force %(local_directory)s
+          fi
+          """)
+      rdiff_wrapper_content = rdiff_wrapper_template % {
+              'comment': comments,
+              'rdiffbackup_binary': self.options['rdiffbackup-binary'],
+              'local_directory': local_directory,
+              'rdiffbackup_parameter': ' \\\n    '.join(rdiffbackup_parameter_list),
+              # XXX: only 10 increments is not enough by default.
+              'remove_backup_older_than': entry.get('remove-backup-older-than', '3B')
+      }
+      rdiff_wrapper = self.createFile(
+          name=rdiff_wrapper_path,
+          content=rdiff_wrapper_content,
+          mode=0700
+      )
 
-    if entry['type'] == 'push':
-      parameters.extend(['--restore-as-of', 'now'])
-      parameters.append('--force')
-      parameters.extend([local_directory, remote_directory])
-      comments = ['','Push data to a PBS *-import instance.','']
-    else:
-      parameters.extend([remote_directory, local_directory])
-      comments = ['','Pull data from a PBS *-export instance.','']
+    path_list.append(rdiff_wrapper)
 
-    wrapper_basepath = os.path.join(self.options['wrappers-directory'],
-                                    url_hash)
-
-    if 'notify' in entry:
-      wrapper_path = wrapper_basepath + '_raw'
-    else:
-      wrapper_path = wrapper_basepath
-
-    wrapper = self.createWrapper(name=wrapper_path,
-                                 command=self.options['rdiffbackup-binary'],
-                                 parameters=parameters,
-                                 comments = comments)
-    path_list.append(wrapper)
-
-    if 'notify' in entry:
-      feed_url = '%s/get/%s' % (self.options['notifier-url'],
-                                entry['notification-id'])
-      wrapper = self.createNotifier(notifier_binary=self.options['notifier-binary'],
-                                    wrapper=wrapper_basepath,
-                                    executable=wrapper_path,
-                                    log=os.path.join(self.options['feeds'], entry['notification-id']),
-                                    title=entry.get('title', 'Untitled'),
-                                    notification_url=entry['notify'],
-                                    feed_url=feed_url,
-                                  )
-      path_list.append(wrapper)
-      #self.setConnectionDict(dict(feed_url=feed_url), entry['slave_reference'])
+    # Create notifier wrapper
+    notifier_wrapper = self.createNotifier(
+        notifier_binary=self.options['notifier-binary'],
+        wrapper=notifier_wrapper_path,
+        executable=rdiff_wrapper,
+        log=os.path.join(self.options['feeds'], entry['notification-id']),
+        title=entry.get('title', slave_id),
+        notification_url=entry['notify'],
+        feed_url='%s/get/%s' % (self.options['notifier-url'], entry['notification-id']),
+        pidfile=os.path.join(self.options['run-directory'], '%s.pid' % slave_id)
+    )
+    path_list.append(notifier_wrapper)
 
     if 'on-notification' in entry:
       path_list.append(self.createCallback(str(entry['on-notification']),
-                                           wrapper))
+                                           notifier_wrapper))
     else:
-      cron_entry = os.path.join(self.options['cron-entries'], url_hash)
+      cron_entry = os.path.join(self.options['cron-entries'], slave_id)
       with open(cron_entry, 'w') as cron_entry_file:
-        cron_entry_file.write('%s %s' % (entry['frequency'], wrapper))
+        cron_entry_file.write('%s %s' % (entry['frequency'], notifier_wrapper))
       path_list.append(cron_entry)
 
     return path_list
@@ -194,7 +234,6 @@ class Recipe(GenericSlapRecipe, Notify, Callback):
       slaves = json.loads(self.options['slave-instance-list'])
       known_hosts = KnownHostsFile(self.options['known-hosts'])
       with known_hosts:
-        # XXX this API could be cleaner
         for slave in slaves:
           path_list.extend(self.add_slave(slave, known_hosts))
     else:
@@ -209,4 +248,3 @@ class Recipe(GenericSlapRecipe, Notify, Callback):
       path_list.append(wrapper)
 
     return path_list
-
